/*----------------------------------------------------------------------------*/
/* generate.c - create a new generation of individuals, with OPTION to use a  */
/*            "crowding factor", similar (not identical) to DeJong '75, as    */
/*            described in Goldberg, p. 190.  Also allows OPTIONAL use of     */
/*            "incest reduction" (developed by Erik Goodman) as a part        */
/*            of the selection process, which tends to favor crossovers among */
/*            relatively fit individuals which are relatively different from  */
/*            each other (in Hamming distance).  It does this by picking the  */
/*            first parent as usual according to its fitness, and picking the */
/*            second parent from among n candidates chosen according to their */
/*            fitness, as usual.  However, it uses the one which is FURTHEST  */
/*            in Hamming distance from the first parent, thus helping to put  */
/*            together genotypes which are both fit, but for "different       */
/*            reasons."  However, the "pool" of individuals (from oldpop) who */
/*            will survive and/or reproduce is NOT altered, and all of the    */
/*            individuals in that pool will be used in SOME crossover, or     */
/*            mutate or survive unaltered into the next generation.  Uses any */
/*            type of selection, etc.                                         */
/*     Also contains closest(), which calculates Hamming distances between    */
/*     one chromosome and several others in array, returning index of closest */
/*     Also contains remap_then_objfunc()                                     */
/*----------------------------------------------------------------------------*/
#include "stdafx.h"
#include "MSetar.h"
#include "MSetarDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#include "external.h"

/* temporary, until I/O stabilizes, then put in global and startup.c */
static int last, j, k;
static int match;
static int mustinsertbest;
static double bestguyfitness;


void generation() {

	/* Determine whether or not to perform inversion on the whole   */
    /* subpopulation before start of this generation (pinversion is */
    /* inversion probability PER GENERATION).                       */


    /* Decide whether is crowding model or not... if not,    */
    /* proceed with usualgen.  If crowding on, proceed with crowder. */

    if (crowding_factor != 0)
		crowder_with_optional_incest_reduction();
    else
		usualgen();


    if(elitism) {
		mustinsertbest = 1;
		bestguyfitness = oldpop[bestnow].init_fitness;
		j = 0;
		while (j < popsize && mustinsertbest == 1) {
			// Necessary, but not sufficient...must check chrom identity 
			if(newpop[j].init_fitness == bestguyfitness)    { // controlla prima se hanno la stessa fitness
				match = 1;
				for (k = 0; k < chromsize; k++) // controlla se i cromosomi sono uguali (bit per bit)
					match = match && (newpop[j].chrom[k] == oldpop[bestnow].chrom[k]);
				mustinsertbest = !match;
			}
			j++;
		}

		if(mustinsertbest) {
			j = rnd(0, popsize - 1);
			for (k = 0; k < chromsize; k++) newpop[j].chrom[k] = oldpop[bestnow].chrom[k];
			newpop[j].parent[0] = bestnow + 1;  // +1 for external printing 
			newpop[j].xsite[0] = 0;
			newpop[j].xsite[1] = 0;
			newpop[j].parent[1] = 0;
			newpop[j].init_fitness = oldpop[bestnow].init_fitness;
			newpop[j].neval = oldpop[bestnow].neval;
			newpop[j].fitness = oldpop[bestnow].fitness; // unneeded but safe 
			sys_util_copy_utility(newpop[j].utility, oldpop[bestnow].utility);
		}
	}

    //Call routine to copy utility fields to survivors not evaluated this 
    // generation, if utility fields are in use. 
    sys_util_survive_without_eval();  

    //call app-dependent generation routine to do any app-specific work 
    // needed to complete generation process  
    app_generate();

  
}

/* Call genetic operators using a crowding model and optional incest reduction */
/* For crowding, first use user's chosen selection to find indices             */
/* in oldpop of preliminary survivors for newpop, but with NO CROSSOVER        */
/* or mutation yet.                                                            */

void crowder_with_optional_incest_reduction() {

    int     mate1, mate2, i, ii, kk, k, jj, j;
    int     distance, furthest_distance, possmate2;
    int     jcross[2];
    int     npairs, killed;
    long    opcounttemp;
    float   remainder, crosssave;

 
    // perform any preselection actions necessary before generation (call preselect()) to prepare for 
	// preliminary selection of those to mate or survive                                             
    if(type_selection==0) preselect_rank();
    if(type_selection==1) preselect_roulette_wheel();
	if(type_selection==2) preselect_SUS();
    if(type_selection==3) preselect_tournament();
	if(type_selection==4) preselect_stochastic_remainder();


    // selection for survival (unless replaced by crossover or altered by 
    // mutation when they occur later).   
    // Note on indices:  i indexes next spot in newpop to fill; j indexes 
    // newpop; jj indexes oldpop; jj = available[j] holds place in oldpop 
    // from which chromosome should be copied to newpop[j]; so entries 
    // should look either like oldpop[jj] or newpop[i] or newpop[killed]  
    

	if(type_selection==0) {
		for (j = 0; j < popsize; j++) {
			available[j] = selection_rank();
			evaluate[j] = 0; 
		}//zero the "must evaluate" flags 
    }

    if(type_selection==1) {
		for (j = 0; j < popsize; j++) {
			available[j] = selection_roulette_wheel();
			evaluate[j] = 0;
		}// zero the "must evaluate" flags 
    }

	if(type_selection==2) {
		for (j = 0; j < popsize; j++) {
			available[j] = selection_SUS();
			evaluate[j] = 0;
		}// zero the "must evaluate" flags 
    }

	if(type_selection==3) {
		for (j = 0; j < popsize; j++) {
			available[j] = selection_tournament();
			evaluate[j] = 0;
		}// zero the "must evaluate" flags 
    }



	if(type_selection==4) {
		for (j = 0; j < popsize; j++) {
			available[j] = selection_stochastic_remainder();
			evaluate[j] = 0;
		}// zero the "must evaluate" flags 
    }



    // (Now the array available[] contains the indices of the individuals 
    // in oldpop[] which will survive or produce offspring into newpop,   
    // unless they are first eliminated by having their slot taken by a   
    // child of a crossover performed before  their  index  is chosen for 
    // reproduction from the available[] list.)  Note that as individuals 
    // are "used", their entries are removed from  available[].           
    last = popsize - 1;


    // Now figure out how many pairs must be crossed over   
    npairs = (int) (pcross * popsize / 2.0);    // note truncation 
    remainder = (pcross * popsize / 2.0) - npairs;
    if (flip(remainder)) npairs++;       //now npairs is (stochast.) correct number  
    k = 2 * npairs;
    // now FORCE crossover to actually do the crosses it is called for 
    crosssave = pcross;
    pcross = 1.0;

    // Fill the first k slots of newpop with the results of  crossover, but 
    // remove OTHER survivors, not necessarily the first k, from the newpop 
    // list by removing their indices from available[].                     

    for (i = 0; i < k; i += 2) {
		mate1 = available[rnd(0, last)];
		if (incest_reduction > 1) {
			// First, find the furthest of incest_reduction possible 
	        // mate2's from mate1 
	        furthest_distance = 0;
	        mate2 = mate1; // Will be repl. by any possmate2 diff. from mate1 
	        for (ii = 0;ii<incest_reduction;ii++) {
				possmate2 = available[rnd(0, last)];
				if ((distance = hamming(&(oldpop[possmate2]) , &(oldpop[mate1]))) > furthest_distance)   {
					furthest_distance = distance;
					mate2 = possmate2;
				}
			}
		}
		else {
			// Using crowding, but not incest reduction 
	        mate2 = available[rnd(0, last)];
		}

		// Bug in earlier releases fixed in Release 2.20:  individuals to be crossed 
        // over were sampled uniformly from oldpop for crossover, not according to   
        // the selection distribution (in available[]), so crowding did not work     
        // properly.  Fixed now by choosing them uniformly from available[], as was  
        // originally intended, but miscoded and not detected in Release 2.05.       

	    if(type_crossover==0)
			crossover_oneptx(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==1)
			crossover_twoptx(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==2)
			crossover_uniform(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==3)
			crossover_cycle(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==4)
			crossover_uniform_order(oldpop[mate1].chrom,oldpop[mate2].chrom,kidholder[0].chrom,kidholder[1].chrom,jcross);
		if(type_crossover==5)
			crossover_pmx(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==6)
			crossover_order(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);
		if(type_crossover==7)
			crossover_app(oldpop[mate1].chrom, oldpop[mate2].chrom, kidholder[0].chrom, kidholder[1].chrom, jcross);

		// Call Hamming comparator fn to get index of chromosome closest 
	    // to the "kids."   Upon return, available[killed] is index in   
	    // newpop of guy to be replaced by crossover child 0.
	    killed = closest(0);

	   // closest compares kidholder[argument] with crowding_factor kids    
	   // still left in available[], between 0 and last, from which it draws
	   // at unif. random.  Returns index (in newpop) of closest guy.       
	   // Then the appropriate child can be actually copied into newpop.    
	   // Available[] list must be updated so can't replace the same newpop 
	   // individual twice. 

	   for (kk = 0; kk < chromsize; kk++) newpop[i].chrom[kk] = kidholder[0].chrom[kk];

	   evaluate[i] = 1;
	   available[killed] = available[last--];
	   newpop[i].parent[0] = mate1 + 1;
	   newpop[i].parent[1] = mate2 + 1;
	   newpop[i].xsite[0] = jcross[0];
	   newpop[i].xsite[1] = jcross[1];

	  // Now repeat same sort of thing for child 1 
	  killed = closest(1);
	  for (kk = 0; kk < chromsize; kk++) newpop[i + 1].chrom[kk] = kidholder[1].chrom[kk];

	  evaluate[i + 1] = 1;
	  available[killed] = available[last--];
	  newpop[i + 1].parent[0] = mate1 + 1;
	  newpop[i + 1].parent[1] = mate2 + 1;
	  newpop[i + 1].xsite[0] = jcross[0];
	  newpop[i + 1].xsite[1] = jcross[1];
	}

	pcross = crosssave; // Restore true crossover rate  


    // Now copy rest of survivors (NOT replaced by crossover) into newpop 
    for (j = k; j < popsize; j++) {
		jj = available[j - k];
		for (kk = 0; kk < chromsize; kk++) newpop[j].chrom[kk] = oldpop[jj].chrom[kk];
	    // then set rest of values as approp. 
	    newpop[j].parent[0] = jj + 1;   // "external" nos. 1 > than internal 
	    newpop[j].parent[1] = 0;
	    newpop[j].xsite[0] = 0;
	    newpop[j].xsite[1] = 0;
	   // Note:  must also copy this info below, because objfunc WON'T be 
	   // called to fill it in, as it is after mutation or crossover. 
	   // Actually, might still be mutated, so evaluated, but logic is  
	   // simplified doing it here, and letting objfunc overwrite if it 
	   // is called after mutation.                                     
	   newpop[j].neval = oldpop[jj].neval;
	   newpop[j].init_fitness = oldpop[jj].init_fitness;
	   newpop[j].fitness = oldpop[jj].fitness;
	   sys_util_copy_utility(newpop[j].utility, oldpop[jj].utility);
    }

    // Then mutate NEWPOP in place, tracking in evaluate 
    for (j = 0; j < popsize; j++) {
		opcounttemp = nmutation;
		if (permproblem == 0 && alpha_size != 2) {

			if(type_mutation==0)      field_mutation_bit(newpop[j].chrom);
			if(type_mutation==1)      field_mutation_multiple(newpop[j].chrom);
			if(type_mutation==2)      field_mutation_scramble(newpop[j].chrom);
			if(type_mutation==3)      field_mutation_swap(newpop[j].chrom);
			if(type_mutation==4)      field_mutation_app(newpop[j].chrom); 
		}
		else  {
			if(type_mutation==0)      mutation_bit(newpop[j].chrom);
			if(type_mutation==1)      mutation_multiple(newpop[j].chrom);
			if(type_mutation==2)      mutation_scramble(newpop[j].chrom);
			if(type_mutation==3)      mutation_swap(newpop[j].chrom);
			if(type_mutation==4)      mutation_app(newpop[j].chrom); 
		}
		if (nmutation != opcounttemp)            evaluate[j] = 1;    // must evaluate 
    }

    // Then evaluate all that changed, bump counters, etc. 
    for (j = 0; j < popsize; j++)
		if (evaluate[j] || stochastic) {
			remap_then_objfunc(&(newpop[j]));
		}



}

/* Perform ordinary crossover and mutation, with kids replacing parents */
void usualgen() {

	int     mate1, mate2, j = 0;
    int     jcross[2];
    long    opcounttemp;
    int     cross_flag;

    /* perform any preselection actions necessary before generation */
    if(type_selection==0) preselect_rank();
    if(type_selection==1) preselect_roulette_wheel();
	if(type_selection==2) preselect_SUS();
    if(type_selection==3) preselect_tournament();
	if(type_selection==4) preselect_stochastic_remainder();

    /* select, crossover, and mutation */
    do {

		/* pick a pair of mates */
		if(type_selection==0) mate1 = selection_rank();
		if(type_selection==1) mate1 = selection_roulette_wheel();
	    if(type_selection==2) mate1 = selection_SUS();
        if(type_selection==3) mate1 = selection_tournament();
	    if(type_selection==4) mate1 = selection_stochastic_remainder();

		if(type_selection==0) mate2 = selection_rank();
		if(type_selection==1) mate2 = selection_roulette_wheel();
	    if(type_selection==2) mate2 = selection_SUS();
        if(type_selection==3) mate2 = selection_tournament();
	    if(type_selection==4) mate2 = selection_stochastic_remainder();
		

		/* Crossover and mutation */
		cross_flag = 0;
		evaluate[j] = 0;
		evaluate[j + 1] = 0;    /* no eval needed unless xover or mutation done.  */
		opcounttemp = ncross;

	    if(type_crossover==0)
			crossover_oneptx(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==1)
			crossover_twoptx(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==2)
			crossover_uniform(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==3)
			crossover_cycle(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==4)
			crossover_uniform_order(oldpop[mate1].chrom,oldpop[mate2].chrom,newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==5)
			crossover_pmx(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==6)
			crossover_order(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);
		if(type_crossover==7)
			crossover_app(oldpop[mate1].chrom, oldpop[mate2].chrom, newpop[j].chrom, newpop[j + 1].chrom, jcross);

		// Ovviamente opcounttemp sar diverso da ncross ogni volta che avviene il crossover
		if(opcounttemp != ncross) {
			cross_flag = 1;
			evaluate[j] = 1;
			evaluate[j + 1] = 1;        /* must evaluate both children */
		}
        
		//Mutazione
		opcounttemp = nmutation;
		if (permproblem == 0 && alpha_size != 2){

			if(type_mutation==0)      field_mutation_bit(newpop[j].chrom);
			if(type_mutation==1)      field_mutation_multiple(newpop[j].chrom);
			if(type_mutation==2)      field_mutation_scramble(newpop[j].chrom);
			if(type_mutation==3)      field_mutation_swap(newpop[j].chrom);
			if(type_mutation==4)      field_mutation_app(newpop[j].chrom); 
		}
			
		else{
			if(type_mutation==0)      mutation_bit(newpop[j].chrom);
			if(type_mutation==1)      mutation_multiple(newpop[j].chrom);
			if(type_mutation==2)      mutation_scramble(newpop[j].chrom);
			if(type_mutation==3)      mutation_swap(newpop[j].chrom);
			if(type_mutation==4)      mutation_app(newpop[j].chrom); 
		}
			

		if (nmutation != opcounttemp)
			evaluate[j] = 1;    /* must evaluate child 1 */
		
		opcounttemp = nmutation;
		if (permproblem == 0 && alpha_size != 2){
			if(type_mutation==0)      field_mutation_bit(newpop[j+1].chrom);
			if(type_mutation==1)      field_mutation_multiple(newpop[j+1].chrom);
			if(type_mutation==2)      field_mutation_scramble(newpop[j+1].chrom);
			if(type_mutation==3)      field_mutation_swap(newpop[j+1].chrom);
			if(type_mutation==4)      field_mutation_app(newpop[j+1].chrom); 
		}
			
		else {
			if(type_mutation==0)      mutation_bit(newpop[j+1].chrom);
			if(type_mutation==1)      mutation_multiple(newpop[j+1].chrom);
			if(type_mutation==2)      mutation_scramble(newpop[j+1].chrom);
			if(type_mutation==3)      mutation_swap(newpop[j+1].chrom);
			if(type_mutation==4)      mutation_app(newpop[j+1].chrom); 
		}
			

		if (nmutation != opcounttemp)
			evaluate[j + 1] = 1;        /* must evaluate child 2 */

		/* Decode string, evaluate fitness, & record */
		/* parentage data on both children */
		if (cross_flag == 1) {  /* evaluate both children */
			
			remap_then_objfunc(&(newpop[j]));
			newpop[j].parent[0] = mate1 + 1;
			newpop[j].xsite[0] = jcross[0];
			newpop[j].xsite[1] = jcross[1];
			newpop[j].parent[1] = mate2 + 1;

			remap_then_objfunc(&(newpop[j + 1]));
			newpop[j + 1].parent[0] = mate1 + 1;
			newpop[j + 1].xsite[0] = jcross[0];
			newpop[j + 1].xsite[1] = jcross[1];
			newpop[j + 1].parent[1] = mate2 + 1;
		}
		else {  /* must handle child 1 & 2, dep on mutation or not */
			if (evaluate[j] == 1) {     /* then evaluate child 1 because mutated */
				remap_then_objfunc(&(newpop[j]));
				newpop[j].parent[0] = mate1 + 1;
				newpop[j].xsite[0] = 0;
				newpop[j].xsite[1] = 0;
				newpop[j].parent[1] = 0;
			}
			else {      /* handle child 1 with no evaluation, init_fitness unchanged  */
				newpop[j].parent[0] = mate1 + 1;
				newpop[j].xsite[0] = 0;
				newpop[j].xsite[1] = 0;
				newpop[j].parent[1] = 0;

				if (stochastic)
					remap_then_objfunc(&(newpop[j]));
				else {
					newpop[j].init_fitness = oldpop[mate1].init_fitness;
					newpop[j].neval = oldpop[mate1].neval;
					sys_util_copy_utility(newpop[j].utility, oldpop[mate1].utility);
				}

				/* Next maybe unneeded, but safe. */
				newpop[j].fitness = oldpop[mate1].fitness;
			}

			if (evaluate[j + 1] == 1) { /* eval child 2 because mutated */
				remap_then_objfunc(&(newpop[j + 1]));
				newpop[j + 1].parent[0] = mate2 + 1;
				newpop[j + 1].xsite[0] = 0;
				newpop[j + 1].xsite[1] = 0;
				newpop[j + 1].parent[1] = 0;
			}
			else {
				newpop[j + 1].parent[0] = mate2 + 1;
				newpop[j + 1].xsite[0] = 0;
				newpop[j + 1].xsite[1] = 0;
				newpop[j + 1].parent[1] = 0;
				
				if (stochastic)
					remap_then_objfunc(&(newpop[j + 1]));
				else {
					newpop[j + 1].init_fitness = oldpop[mate2].init_fitness;
					newpop[j + 1].neval = oldpop[mate2].neval;
					sys_util_copy_utility(newpop[j + 1].utility, oldpop[mate2].utility);
				}
				/* Next maybe unneeded, but safe. */
				newpop[j + 1].fitness = oldpop[mate2].fitness;
			}
		}
		/* Increment population index */
		j = j + 2;
	}
	while (j < (popsize - 1));

}

/* For crowding crossover, returns the index (from "available", which indexes */
/* newpop) of individual of crowding_factor picked at uniform random from   */
/* closest to (in Hamming distance) holding_chromosomes[i].  Calls          */
/* hamming(), which is defined in utility.c                                 */

int closest(int i) {

    int     k, j, tokill, availindex, distance, picklimit, mindist;


    mindist = 32767;    /* safe large value for short int */
    picklimit = crowding_factor;
    
	if (last < crowding_factor) picklimit = last + 1;

    for (k = 0; k < picklimit; k++) {
		availindex = rnd(0, last);      /* index in available */
		j = available[availindex];      /* corresp. index in oldpop*/
		if (j > popsize - 1) {
			fprintf(stderr, " error in crowding\n");
			exit(-99);
		}
		distance = hamming(&(oldpop[j]), &(kidholder[i]));
		/*      fprintf(outfp, "availindex=%d, dist=%d\n", availindex, distance); */
		if (distance < mindist) {
			mindist = distance;
			tokill = availindex;
		}
    }
/*  fprintf(outfp, "tokill=%d\n", tokill); */
    return tokill;
}


/* This function just calls the objfunc in app......c, unless     */
/* using inversion, or different_reps, in which cases it calls objfunc with */
/* the chrom remapped into the "standard" form which the objfunc processes. */
void remap_then_objfunc(struct individual *critter) {

    int i;
    unsigned *SaveOriginalChromPointer;

    /* Now evaluate the fitness */
    objfunc(critter);

}


void remap_then_objfunc2(struct individual *critter) {

    int i;
    unsigned *SaveOriginalChromPointer;

    /* Now evaluate the fitness */
    //objfunc_print(critter);

}




/* Copy operation done from parents to children for ALL kids which were NOT */
/* evaluated this generation (i.e., no crossover or mutation was done to */
/* create them, so objfunc was not called.)  So the utility fields, which */
/* are usually filled in by obj_func when a NEW individual is generated, */
/* are copied from the parent here, instead.                   */

void sys_util_survive_without_eval() {

	int     parent, i, j, nbytes;
	unsigned char *destbyte, *sourcebyte;

	for(j = 0; j < popsize; j++) {
		if(evaluate[j] == 0) {
			nbytes = GetUtilitySize();
			if(nbytes) {
				parent = newpop[j].parent[0] - 1; /* subtract 1 for internal indexing */
				destbyte = (unsigned char *) newpop[j].utility;
				sourcebyte = (unsigned char *) oldpop[parent].utility;
				for(i=0;i<nbytes;i++) {
					destbyte[i] = sourcebyte[i];
				}
			}
		}
	}
}


void sys_util_copy_utility(int *destutility, int *sourceutility) {

    
    int     j, nbytes;
    unsigned char *destbyte, *sourcebyte;
    
	nbytes = GetUtilitySize();
    /* Does one byte at a time. */
    destbyte = (unsigned char *) destutility;
    sourcebyte = (unsigned char *) sourceutility;
    for(j=0;j<nbytes;j++) {
	 destbyte[j] = sourcebyte[j];
    }

	
}

